$title	Stochastic Programming Model of Embodied Technical Change

$if not "%plots%"=="none" $batinclude plot 

$if not set taxyear $set taxyear 2010
$if not set singlesw $set singlesw 

set	tp			Time periods including first post-terminal period /2005*2061/,
	t(tp)			Time periods in the model /2005*2060/,
	tlast(tp)		Last endogenous year /2060/,
	tplot(tp)		Time periods to plot /2005*2050/,
	tl(t)			Labels for plotting /2010,2020,2030,2040,2050/
	tfirst(t)		First year in the model /2005/,
	sw			States of world /s0*s10/
	transition(t,sw,sw)	State transitions;

transition("%taxyear%","s0",sw) = yes;
transition("%taxyear%","s0","s0") = no;
display transition;

execute_unload 'probtree.gdx', tp=t, sw, transition;
execute 'gams probtree gdx=treelogic.gdx';

alias (sw,ssw,sow);

set	eq(tp,sw)	Equilibrium structure (sw is active in time period tp),
	st(tp,sw,sow)	State transitions (sw transitions to sow in time period tp),
	sm(tp,sw,sow)	State matching (sw is represented by state sow in time period tp);

execute_load 'treelogic.gdx', eq, st, sm;

set		sl(tp,sw,sow)	States requiring locking constraints;
sl(tp,sw,sow) = yes$(sm(tp,sw,sow) and (not sameas(sw,sow)));

parameter	pi(sw)		Perceived probability;
		
pi(sw) = 1/card(sw);

*	First plot the probability of an innovation event:

$setglobal domain tplot
$setglobal labels tl

execute_unload 'treeplot.gdx', sw, t, pi, sm, eq,  tl;
$if not "%plots%"=="none" execute 'gams treeplot --piscale=no';

parameter	
	r		Interest rate /0.05/
	g		Growth rate   /0.02/

	crra		Coefficient of relative risk aversion /1/,
	esubt		Intertemporal elasticity of substitution /0.5/
	sigma(sw)	Capital-labor elasticity of substitution 
	delta		Capital depreciation rate /0.07/
	y0		Base year output /100/
	l0		Base year labor supply /60/

	pref(tp)	Baseline present value price,
	qref(tp)	Baseline growth index,

	tk(t)		Capital tax rate

	rk0		Base year capital earnings,
	u0		Baseline utility index,
	c0		Base year consumption,
	i0		Base year investment,
	k0		Base year capital stock,
	phi(tp,sw)	Investment productivity;

rk0 = y0 - l0;
k0 = rk0 / (r + delta);
i0 = (g+delta)*k0;
c0 = y0-i0;
phi(tp,sw) = 1;

sigma(sw) = normal(1,0.25);
sigma("s0") = 1;

pref(tp) = (1/(1+r))**(ord(tp)-1);
qref(tp) = (1+g)**(ord(tp)-1);
tk(t) = 0;

u0 = sum(t, pref(t)*qref(t)*c0);

$ontext

$model:stc

$sectors:
	u(sw)				! Utility
	y(t,sw)$eq(t,sw)		! Output
	i(tp,sw)$(t(tp) and eq(tp,sw))	! Investment
	k(tp,sw)$(t(tp) and eq(tp,sw))	! Capital
	c(t,sw)$eq(t,sw)		! Consumption

$commodities:
	pu(sw)				! Welfare
	pc(t,sw)			! Output
	p(t,sw)$eq(t,sw)		! Output
	rk(tp,sw)$(t(tp) and eq(tp,sw))	! Return
	pk(tp,sw)$eq(tp,sw)		! Capital price
	pl(t,sw)$eq(t,sw)		! Wage rate

$consumers:
	ra

$auxiliary:
	kt(sw)		! Terminal capital stock

$prod:u(sw)  s:esubt
	o:pu(sw)	q:u0
	i:pc(t,sw)	q:(qref(t)*c0)	p:pref(t)

$prod:y(t,sw)$eq(t,sw)  s:sigma(sw)
	o:p(t,sw)	q:y0	
	i:pl(t,sw)	q:l0
	i:rk(t,sw)	q:rk0	a:ra	t:tk(t)

$prod:k(tp,sw)$(t(tp) and eq(tp,sw))
	o:pk(tp+1,sow)$st(tp,sw,sow)	q:(k0*(1-delta))
	o:rk(tp,sw)			q:rk0	
	i:pk(tp,sw)			q:k0

$prod:i(tp,sw)$(t(tp) and eq(tp,sw))
	o:pk(tp+1,sow)$st(tp,sw,sow)	q:i0
	i:p(tp,sw)			q:i0
	
$prod:c(t,sw)$eq(t,sw)  t:0
	o:pc(t,sow)$sm(t,sow,sw)	q:c0	
	i:p(t,sw)			q:c0

$demand:ra  s:crra
	d:pu(sw)			q:u0	p:pi(sw)
	e:pk(tfirst,sw)$eq(tfirst,sw)	q:k0	
	e:pl(t,sw)$eq(t,sw)		q:(l0*qref(t))
	e:pk(tp,sw)$tlast(tp-1)		q:(-k0)		r:kt(sw)

$constraint:kt(sw)
	sum(tlast(t), i(t,sw)/i(t-1,sw) - c(t,sw)/c(t-1,sw)) =e= 0;


$offtext
$sysinclude mpsgeset stc

u.l(sw) = 1;
y.l(t,sw) = qref(t);
i.l(t,sw) = qref(t);
k.l(t,sw) = qref(t);
c.l(t,sw) = qref(t);
pu.l(sw) = pi(sw);

pc.l(t,sw) = pref(t)*pi(sw);
p.l(t,sw)   = pref(t) * sum(sm(t,sow,sw), pi(sow));
rk.l(t,sw)  = pref(t) * sum(sm(t,sow,sw), pi(sow));
pk.l(tp,sw) = pref(tp)* sum(sm(tp,sow,sw), pi(sow)) * (1+r);
pl.l(t,sw)  = pref(t) * sum(sm(t,sow,sw), pi(sow));

kt.l(sw) = sum(tlast(tp), qref(tp)*(1+g));

stc.workspace = 25;
stc.iterlim = 0;

$include stc.gen
solve stc using mcp;
stc.iterlim = 20000;

loop(t,
	tk(t)$(tk(t-1)) = 0.3;
	tk(t)$sameas(t,"%taxyear%") = 0.3;
);
display tk;

*	If we are solving a deterministic case, all scenarios
*	have the same elasticity:

$if not "%singlesw%"=="" sigma(sw) = sigma("%singlesw%");

$include stc.gen
solve stc using mcp;

parameter	wage	Wage rate in equilibrium
		return	Rental rate	
		invest	Investment level
		output	Output level;

loop(sm(t,sw,sow),
	wage(t,sw) = pl.l(t,sow)/p.l(t,sow);
	return(t,sw) = rk.l(t,sow)/p.l(t,sow);
	invest(t,sw) = i.l(t,sow)/qref(t);
	output(t,sw) = y.l(t,sow)/qref(t);
);

$if "%plots%"=="none" $exit

$setglobal domain tplot
$setglobal labels tl
$setglobal gp_opt0 'unset key'
$batinclude plot wage
$batinclude plot return
$batinclude plot invest
$batinclude plot output

execute_unload 'fan.gdx',tplot=t,sw=s,tl,wage,return,invest,output,pi=prob;
execute 'gams fan --item=wage';
execute 'gams fan --item=return';
execute 'gams fan --item=invest';
execute 'gams fan --item=output';

