$title A Stochastic Equilibrium Model in GAMS -- Illustrating use of the SP Tools

*	"Mixed Complementarity Formulations of Stochastic 
*	Equilibrium Models with Recourse"

*	Alex Meeraus
*	Thomas Rutherford

*	October, 2005

*	This program illustrates use of the following GAMS libinclude utilities
*	which permit tight formulations of stochastic programming models in
*	GAMS (i.e., model representations which do not rely on non-anticipativity
*	constraints):

*		plot		GAMS code for generating simple line plots
*				with GNUPLOT 4.1
*		probtree	GAMS code for constructing logical tuples 
*				for efficient stochastic-programming.
*		treeplot	GAMS code for visualization of a stochastic
*				programming event tree using GNUPLOT 4.1.
*		fan		GAMS code for generating probability 
*				distribution fan plot diagrams with
*				GNUPLOT 4.1.
*
*	These libinclude routines rely on the following external programs:
*
*		wgnuplot.exe	GNUPLOT executable, version 4.1
*		wgnuplot.mnu	Additional programs in the GNUPLOT distribution	
*		wgnuplot.hlp
*		wgnuplot.GID
*		gdxrank.exe	Utility routine for ranking parameter arrays
*				in GAMS GDX files.
*		gdxio.dll	Utility DLL used by GDXRANK.
*		
*	-----------------------------------------------------------------
*	NB	If you wish to have these utilities as a permanent 
*		component of you GAMS installation, copy the .gms 
*		utilities to your GAMS inclib directory, and copy the
*		wgnuplot files to your GAMS system directory. (gdxrank.exe
*		and gdxio.dll are distributed with GAMS).
*
*		After having copies files, replace $batinclude by
*		$libinclude.
*	-----------------------------------------------------------------

*	Define the time dimension of the mode in arrays tp and t.  

set	tp		Time periods with one post-terminal period /2005*2101/,
	t(tp)		Time periods in the model /2005*2100/,
	tfirst(t)	First year in the model,
	tlast(tp)	Last endogenous year,

	sw			States of world /2010*2030,never/
	transition(t,sw,sw)	State transitions

*	Define the event tree by specifying which states generate transitions.
*	In this case sw1 defines the root node of the tree.  State 9 is the 
*	the first transition in year 2010.
*	
		/ 2010.2010.2011 /

loop((t,sw)$transition(t,sw,sw+1),transition(t+1,sw+1,sw+2) =yes;);
display transition;


execute_unload 'probtree.gdx', tp=t, sw, transition;

execute 'gams probtree gdx=treelogic.gdx';


tfirst(t) = yes$(ord(t) eq 1);
tlast(t) = yes$(ord(t)  eq card(t));

alias (sw,ssw,sow);

set	eq(tp,sw)	Equilibrium structure (sw is active in time period tp),
	st(tp,sw,sow)	State transitions (sw transitions to sow in time period tp),
	sm(tp,sw,sow)	State matching (sw is represented by state sow in time period tp);

execute_load 'treelogic.gdx', eq, st, sm;


set	sl(tp,sw,sow)	States requiring locking constraints;
sl(tp,sw,sow) = yes$(sm(tp,sw,sow) and (not sameas(sw,sow)));

parameter	sp(tp,sw)	Preceding state of world;
loop(st(tp,sow,sw),	sp(tp,sw) =  ord(sw)-ord(sow););

parameter	pi(sw)		Period-by-period probability of efficiency improvement,
		var		Variance in probability of event /0.05/;

*	Define the probability of an efficiency improvement using a 
*	normal distribution define over a set of time periods:

pi(sw) = exp(-var * sqr(ord(sw)-card(sw)/2));
pi("never") = 0;
pi(sw) = 0.75 * pi(sw)/sum(sow,pi(sow));
display pi;

set	tplot(tp)	Time periods to plot /2005*2050/,
	tl50(t)		Labels for plotting through 2050 /2010,2020,2030,2040,2050/;

*	Define the domain and labels for the first plot:

$setglobal domain tplot
$setglobal labels tl50

*	Generate the cumulative density function simply for illustration
*	of the model input assumptions:

parameter cdf(*)	Cumulative density function;
cdf(t) = 0;
$onuni
loop(t,
	cdf(t+1) = cdf(t) + pi(t);
);

*	Label the plot every 20 years through 2100:

set t2100(t) /2020,2040,2060,2080,2100/

$setglobal domain t
$setglobal labels t2100
$batinclude plot cdf
$batinclude plot pi

*	The probability of no technology change is one minus the probability
*	of an innovation at some point during the specified time periods:

pi("never") = 1 - sum(sw,pi(sw));
display pi;


*	Generate a figure illustrating the stochastic structure in which the
*	spacing of the terminal nodes is proportional to the probability of
*	those outcomes:

set	tl(t)		Labels for plotting /2020,2040,2060,2080,2100/;

execute_unload 'treeplot.gdx', sw, t, pi, sm, eq,  tl;
execute 'gams treeplot --piscale=yes';

parameter	
	r		Interest rate /0.05/
	g		Growth rate   /0.02/

	pref(tp)	Baseline present value price
	qref(tp)	Baseline growth index

	delta		Capital depreciation rate /0.07/
	y0		Base year output /100/
	l0		Base year labor supply /60/
	le(tp,sw)	Labor endowments
	rk0		Base year capital earnings /40/
	u0		Baseline utility index
	c0		Base year consumption
	i0		Base year investment
	k0		Base year capital stock
	phi(tp,sw)	Investment productivity;

k0 = rk0 / (r + delta);
i0 = (g+delta)*k0;
c0 = y0-i0;
phi(tp,sw) = 1;

pref(tp) = (1/(1+r))**(ord(tp)-1);
qref(tp) = (1+g)**(ord(tp)-1);

u0 = sum(t, pref(t)*qref(t)*c0);

parameter
	grate(sw)	Growth rate by state of world
	index(sw)	Numeric index for a given state /2010 1/,
	indexno		Counter for index assignments /1/;

loop(st(tp,sw,sow)$(not sameas(sw,sow)), index(sow) = index(sw) + 1;);
grate(sw) = 0.03 - 0.01 * power(-1, index(sw));
display grate;

parameter
	al(t,sw)	Labor productivity factor
	b		Capital value share
	beta(tp)	Utility discount factor;

beta(t)  = ((1+g)/(1+r))**(ord(t)-1);
beta(tlast) = beta(tlast)/(1-(1+g)/(1+r));
b = rk0 / y0;
al(t,sw) = y0 * (1/k0)**b;

variables
	eu		Expected utility
	u(sw)		Utility
	y(t,sw)		Output
	i(tp,sw)	Investment
	k(tp,sw)	Capital
	c(t,sw)		Consumption;

positive variable i;

equations	expectedu, utility, capital, terminv, market;

*	Write out the equilibrium problem using the sm() array to only reference
*	"active" states of the world in each period:

expectedu..			eu =e= 1000 * sum(sw, pi(sw) * u(sw));

*	eq(t,sw)	Indicates that state sw is active in time period t:

market(eq(t,sw))..		al(t,sw) * qref(t)**(1-b) * k(t,sw)**b =e= c(t,sw) + i(t,sw);

*	sm(t,sw,sow)	Indicates that in year t state sw actions are represented by the active state sow:

utility(sw)..			u(sw) =e= sum(sm(t,sw,sow), beta(t) * log(c(t,sow)));

*	sp(t,sw)	Pointer to scenario generating this scenario in preceding period.

capital(eq(t+1,sw))..		k(t+1,sw) =e= (1-delta)*k(t,sw-sp(t,sw)) + i(t,sw-sp(t,sw)) * phi(t,sw);

*	Terminal investment constraint:

terminv(tlast,sw)..		i(tlast,sw) =g= (g+delta) * k(tlast,sw);

model stc /all/;

*	Assign defaultvalues:

u.l(sw) = 1;
i.l(t,sw) = i0 * qref(t);
k.l(t,sw) = k0 * qref(t);
c.l(t,sw) = c0 * qref(t);

*	Install some lower bounds to avoid bad function calls:

k.lo(t,sw) = k0 * qref(t) / 1000;
c.lo(t,sw) = c0 * qref(t) / 1000;
u.lo(sw) = 0.00001;
k.fx(tfirst,sw) = k0;

*	Scale the model to improve robustness (not required for this model,
*	but this is a good habit):

i.scale(t,sw) = i0 * qref(t);
k.scale(t,sw) = k0 * qref(t);
c.scale(t,sw) = c0 * qref(t);
market.scale(t,sw) = (c0+i0)*qref(t);
capital.scale(t,sw) = k0*qref(t);
stc.scaleopt = 1;

*	Solve the stochastic program with no probability shock:


solve stc using nlp maximizing eu;

*	Define the productivity shock:

phi(tp,sw)$eq(tp,sw) = 1.1;
phi(tp,sw)$(not eq(tp-1,sw)) = 1;
set tpearly(tp)		Early periods - before technology shock /2005*2010/;
phi(tpearly,"2010") = 1;
phi(tp,"never") = 1;
display phi

*	Solve the stochastic program with the probability shock:

solve stc using nlp maximizing eu;

*	Generate some reports:

parameter	wage	Wage rate in equilibrium
		return	Rental rate	
		invest	Investment level
		output	Output level;

*	This logic expands the model solution from the "active" state-time
*	periods to all state-time periods:

loop(sm(t,sw,sow),
	wage(t,sw) = (1-b) * al(t,sow) * (k.l(t,sow)/qref(t))**b;
	return(t,sw) = b * al(t,sow)*(qref(t)/k.l(t,sow))**(1-b);
	invest(t,sw) = i.l(t,sow)/(i0*qref(t));
	output(t,sw) = al(t,sow) * qref(t)**(1-b) * k.l(t,sow)**b / qref(t);
);

*	First produce simple line plots of the outputs:

$setglobal domain tplot
$setglobal labels tl50
$setglobal gp_opt0 'unset key'
$batinclude plot wage
$batinclude plot return
$batinclude plot invest
$batinclude plot output

*	Then unload these data to a GDX file, along with the time 
*	domain (t), the state domain (s), the set of plot labels (tl),
*	and the state probability vector (prob)
execute_unload 'fan.gdx',tplot=t,sw=s,tl,pi=prob,wage,return,invest,output;

*	Generate the probability fan plots:

execute 'gams fan --item=output';
execute 'gams fan --item=invest';
execute 'gams fan --item=wage';
execute 'gams fan --item=return';
